open Util;;
open Logic;; (* use <?> operator from Logic and not from Util *)
module Number = Logic.Number;;
open Operators;;


type t = Gt of int * int | Eq of int * int;;

let arith = {Logic.min = 200; neg = true; rat = 6; real = false; minf = false};;

let test sop op a b c = 
 if not ((op a b) = c) then Format.printf "%a %s %a <> %a@\n"
  Number.fprintf a sop Number.fprintf b Number.fprintf c
;;

let check sop a b c = match sop with
 | "+" -> test sop Number.add a b c
 | "-" -> test sop Number.sub a b c
 | "*" -> test sop Number.mul a b c
;;

let check_cp cop a b c = match cop with
 | ">" -> assert (Number.gt a b = c)
 | "=" -> assert (Number.eq a b = c)
;;

let check_eq va a =
 Format.fprintf Format.str_formatter "%a" Number.fprintf va; 
 let x = Format.flush_str_formatter () in
 Format.fprintf Format.str_formatter "%a" Logic.fprintf_a a; 
 let y = Format.flush_str_formatter () in
 assert (x = y);
;;

let test_op s op sop a b = 
 let result = 
 fresh_arith arith >>= fun x ->
 fresh_arith arith >>= fun y ->
 let c = op x y in
 solve ~solver:s ((a <=> x) <&> (b <=> y)) >>= function 
  | None -> (return "no")
  | Some assign -> (
   eval_a x assign >>= fun va ->
   eval_a y assign >>= fun vb ->
   eval_a c assign >>= fun vc ->
   check sop va vb vc;
   check_eq va a;
   check_eq vb b;
   return "yes")
 in
 (*ignore (run result); *)
 let x = run ~obits:max_int result in
 (*(if x = "no" then  *)
   (*Format.printf "number %a or %a cannot be represented\n"  *)
   (*Logic.fprintf_a a Logic.fprintf_a b; *)
  (*else *)
   (*()); *)
 ()
;;

let test_cop s op cop a b = 
 let c = op (fresh a) (fresh b) in
 let result = 
 solve ~solver:s c >>= function 
  | None -> (return "no")
  | Some assign -> (
   eval_a (fresh a) assign >>= fun va ->
   eval_a (fresh b) assign >>= fun vb ->
   eval_p c assign >>= fun vc ->
   check_cp cop va vb vc;
   return "yes")
 in
 let x = run ~obits:max_int result in
 ()
;;

let test_ops s a b = 
 test_op s (<+>) "+" a b;
 test_op s (<->) "-" a b;
 test_op s (<*>) "*" a b;
 test_cop s (<>>) ">" a b;
 test_cop s (<=>) "=" a b;
;;

let test_from_to s a b d = 
 let ns = (List.range a b) in
 let ds = List.range 1 (d + 1) in
 let ns = List.product ns ds in
 let ns = List.map (fun (n,d) -> Number.of_rat n d) ns in
 let ns = List.map constant ns in
 let ss = List.square ns in
 List.iter (fun (a,b) -> test_ops s a b) ss
;;

(*
let cache f tbl k = 
 (if not (Hashtbl.mem tbl k) then
  f >>= fun x -> Hashtbl.add tbl k x; return tbl
 else
  return tbl) >>= fun tbl ->
 return (Hashtbl.find tbl k)
;;

let cache_bool = cache fresh_bool;;
let a_spec = {min=Int.pow 2 10;rat=1;real=1;neg=false;minf=false};;
let b_spec = {min=7;rat=1;real=1;neg=true;minf=false};;
let cache_arith ?(spec = a_spec) tbl = cache (fresh_arith spec) tbl;;
*)

(*
let test n =
 let a = constant (Number.of_int n) in
 let result = 
  fresh_arith arith >>= fun x ->
   solve ~solver:MiniSat ((x <=> a))  >>= function
   | None -> return "shit"
   | Some assign -> 
     (*Format.printf "Assign %a\n" Logic.fprintf_assignment assign; *)
     (*Format.printf "a = %a\n" Logic.fprintf_a a; *)
     (*Format.printf "x = %a\n" Logic.fprintf_a x; *)
     eval_a x assign >>= fun va ->
     Format.printf "%d = %a@\n" n Number.fprintf va;
     return "yes"
  in
  Format.printf "Surprise: %s\n" (Logic.run result);
  (*ignore (Logic.run result); *)
;;
*)

let test () =
 let ppf = Format.std_formatter in
 Format.fprintf ppf "Testing module Logic ...\n";
 (*test_from_to MiniSat ~-5 10 2; *)
 (*test_from_to Yices ~-5 5 1; *)

 (*List.iter test (List.range ~-20 20); *)
(*
*)
 let x0 = constant (Number.of_int ~-1) in
 let x1 = constant (Number.of_int 0) in
 let result = 
 fresh_arith (Logic.int 1000) >>= fun a ->
 (*fresh_arith (Logic.nat 1) >>= fun b ->  *)
 Format.printf "%a max %a=@\n" Logic.fprintf_a x0 Logic.fprintf_a x1;
 let c = a <=> (max x0 x1) in
 solve ~solver:MiniSat (c) >>= function
  | None -> (return "no")
  | Some assign -> (
    eval_a a assign >>= fun vc -> 
     Format.printf "%a = %a\n" fprintf_a a Number.fprintf vc;
   return "yes") ;
 in
 let x = run ~obits:8 result in
 Format.printf "%s\n" x;
 ();
 exit 0;
  
(*
 let btbl = Hashtbl.create 512 in
 let atbl = Hashtbl.create 512 in
 let result = 
 cache_bool btbl 0 >>= fun x ->
 cache_bool btbl 1 >>= fun y ->
 cache_arith atbl 0 >>= fun a ->
 cache_arith atbl 1 >>= fun b ->
 cache_arith atbl 2 >>= fun c ->
 cache_arith atbl 3 >>= fun d ->
 let pow3 x = x <*> x <*> x in
 let f = 
  ((pow3 a <+> pow3 d) <=> (pow3 b <+> pow3 c))
  <&> (zero <<> a) <&> (a <<> b) <&> (b <<> c) <&> (c <<> d) in
 Format.printf "formula: %a\n%!" fprintf_p f;
 solve ~solver:MiniSat f >>= function
  | None -> (return "no")
  | Some assign -> (
    (* eval_p x assign >>= fun z -> *)
     (* Format.printf "%a = %b\n" fprintf_p x z; *)
    (* eval_p y assign >>= fun z -> *)
     (* Format.printf "%a = %b\n" fprintf_p y z; *)
    eval_a a assign >>= fun va -> 
     Format.printf "%a = %a\n" fprintf_a a Number.fprintf va;
   eval_a b assign >>= fun vb -> 
    Format.printf "%a = %a\n" fprintf_a b Number.fprintf vb;
   eval_a c assign >>= fun vc -> 
    Format.printf "%a = %a\n" fprintf_a c Number.fprintf vc;
   eval_a d assign >>= fun vd -> 
    Format.printf "%a = %a\n" fprintf_a d Number.fprintf vd;
   let pow3 x = Number.mul (Number.mul x x) x in
    Format.printf "%a = %a@\n" 
     Number.fprintf (Number.add (pow3 va) (pow3 vd))
     Number.fprintf (Number.add (pow3 vb) (pow3 vc));
   return "yes")
 in
 let x = run ~obits:11 result in
 Format.printf "%s\n" x;
 ()
 *)
;;

module N = struct
 include Number;;
 let combine = max;;
 let copy = id;;
 let compare = Pervasives.compare
end
module Ngraph = Graph.Labeled (N) (Int)
let test () =
 let l n = Some (Number.of_int n) in
 let g = Ngraph.generate (fun i j -> [(i,l (i+j),j)]) (List.range 0 26) in
 Format.printf "@[<1>G:@\n%a@]@\n%!" Ngraph.fprintf g;
 let rec foo g = 
  if List.length (Ngraph.nodes g) = 1 then () else
  let n = List.hd (Ngraph.nodes g) in
  let g = Ngraph.bypass n g in
  (*Format.printf "@[<1>G:@\n%a@]@\n%!" Ngraph.fprintf g; *)
  Format.printf "%d nodes, %d edges@\n%!" 
   (List.length (Ngraph.nodes g)) (List.length (Ngraph.edges g));
  (*foo g *)
 in
 foo g
;;

let test () = 
 let arith = Logic.int 10 in
 let result = 
 fresh_arith arith >>= fun x ->
 fresh_arith arith >>= fun y ->
 let c = (<*>) x y in
 let phi = (c <=> x) in
 let _ = Format.printf "%a" Logic.fprintf_smt
 phi in
 return ()
 in
 ignore (run result); 
 ()
;;

test ();;
