DIR   := src/ttt2
PACK  := Ttt2
DEP   := Util Parsec Rewriting Logic Automata Processors
GENERATED := input/confParser.ml input/confLexer.ml input/srsParser.ml input/srsLexer.ml input/trsParser.ml input/trsLexer.ml strategy/parser.ml strategy/lexer.ml
FILES := input/wstParser.ml input/trsSyntax.ml input/srsSyntax.ml input/confSyntax.ml input/xml.ml strategy/proof.ml strategy/processor.ml strategy/state.ml strategy/monad.ml strategy/transformation.ml strategy/predicate.ml strategy/modifier.ml strategy/main.ml strategy/nontermination.ml strategy/status.ml strategy/termination.ml strategy/syntax.ml kernel.ml answer.ml $(GENERATED)

include mk/subdir.mk

# take care of generated files
$(BUILD)/%.ml: $(SRC)/%.mly | $$(dir $$@).dir
	$(OCAMLYACC) -b $(@:.ml=) $<
	rm $(@:.ml=.mli)

$(BUILD)/%.ml: $(SRC)/%.mll | $$(dir $$@).dir
	$(OCAMLLEX) -o $@ $<

.PRECIOUS: $(GENERATED:%=$(BUILD)/%)

# build ttt2
# packs to include in ttt2, in dependency order
PACKS_Ttt2 := Util Parsec Rewriting Logic Automata Processors Ttt2

LIBS_Ttt2 = $(foreach x,$(PACKS_Ttt2),$(LIB_$(x)))

# using secondary expansion (the $$) is important: some LIB_Foo variables may
# not be defined yet the first time we get here
ttt2: $$(LIBS_Ttt2) $(DIR)/main.cmx
	$(OCAMLOPT) $(LINK) $(LINK_Logic) -o $@ $^

$(DIR)/main.cmx: $(DIR)/main.ml $(LIB_Ttt2)
	$(OCAMLOPT) -c -I $(DIR_Ttt2) -o $@ $<

# extra housekeeping
clean-$(PACK)::
	rm -f $(addprefix $(DIR_$(PACK))/main,.cmx .cmo .cmi .o)

distclean-$(PACK)::
	rm -rf ttt2

# HACK:
# The build system overapproximates dependencies, which results in circular
# dependencies. Here we identify some false positives to break cycles:

# - uses of Monad from Util
# - uses of Modifier, Nontermination, Terminaton, Transformation from Processors
$(DEPDIR)/strategy/processor.ml: BLACKLIST := Monad Modifier Nontermination Termination Transformation
$(DEPDIR)/strategy/proof.ml: BLACKLIST := Monad Transformation
$(DEPDIR)/strategy/state.ml: BLACKLIST := Monad
$(DEPDIR)/input/%: BLACKLIST := Monad
